@@ -32,7 +32,13 @@
 
 /* Since 3.8 it is mandatory to use proper type for # formats */
 #define PY_SSIZE_T_CLEAN
+
+/* Simplify passing strings to the various compilers*/
+#define str(x) #x
+#define xstr(x) str(x)
+
 #include <Python.h>
+#include "pythoncapi_compat.h"    // Py_SET_SIZE() for Python 3.8 and older
 
 #include "bytesobject.h"
 #include "zstd.h"
@@ -87,8 +93,10 @@ static PyObject *py_zstd_compress_mt(PyObject* self, PyObject *args)
     if (0 == threads) threads = UTIL_countPhysicalCores();
     /* If threads more than 200 - raise Error. */
     if (threads > ZSTDMT_NBWORKERS_MAX) {
-        PyErr_Format(ZstdError, "Bad threads count - more than %d: %d", ZSTDMT_NBWORKERS_MAX, threads);
-        return NULL;
+        threads = ZSTDMT_NBWORKERS_MAX;
+        // do not fail here, due auto thread counter
+        //PyErr_Format(ZstdError, "Bad threads count - more than %d: %d", ZSTDMT_NBWORKERS_MAX, threads);
+        //return NULL;
     }
 
     dest_size = (Py_ssize_t)ZSTD_compressBound(source_size);
@@ -97,7 +105,7 @@ static PyObject *py_zstd_compress_mt(PyObject* self, PyObject *args)
         return NULL;
     }
 
-    if (source_size > 0) {
+    if (source_size >= 0) {
         dest = PyBytes_AS_STRING(result);
 
         cctx = ZSTD_createCCtx();
@@ -116,7 +124,7 @@ static PyObject *py_zstd_compress_mt(PyObject* self, PyObject *args)
             Py_CLEAR(result);
             return NULL;
         }
-        Py_SIZE(result) = cSize;
+        Py_SET_SIZE(result, cSize);
     }
     return result;
 }
@@ -130,9 +138,9 @@ static PyObject *py_zstd_uncompress(PyObject* self, PyObject *args)
 {
 
     PyObject    *result;
-    const char  *source;
-    Py_ssize_t  source_size;
-    uint64_t    dest_size;
+    const char  *source, *src;
+    Py_ssize_t  source_size, ss, seek_frame;
+    uint64_t    dest_size, frame_size;
     char        error = 0;
     size_t      cSize;
 
@@ -144,23 +152,46 @@ static PyObject *py_zstd_uncompress(PyObject* self, PyObject *args)
         return NULL;
 #endif
 
-    dest_size = (uint64_t) ZSTD_getDecompressedSize(source, source_size);
-    if (dest_size == 0) {
+    dest_size = (uint64_t) ZSTD_getFrameContentSize(source, source_size);
+    if (dest_size == ZSTD_CONTENTSIZE_UNKNOWN || dest_size == ZSTD_CONTENTSIZE_ERROR) {
         PyErr_Format(ZstdError, "Input data invalid or missing content size in frame header.");
         return NULL;
     }
+
+	// Find real dest_size across multiple frames
+	ss = source_size;
+	seek_frame = ss - 1;
+	src = source;
+	while (seek_frame < ss) {
+		seek_frame = ZSTD_findFrameCompressedSize(src, ss);
+		if (ZSTD_isError(seek_frame)) break;
+		src += seek_frame;
+		ss -= seek_frame;
+		if (ss <=0) break;
+		frame_size = (uint64_t) ZSTD_getFrameContentSize(src, ss);
+		if (ZSTD_isError(frame_size)) break;
+		dest_size += frame_size;
+	}
+
     result = PyBytes_FromStringAndSize(NULL, dest_size);
 
     if (result != NULL) {
         char *dest = PyBytes_AS_STRING(result);
 
         Py_BEGIN_ALLOW_THREADS
+		// get real dest_size
         cSize = ZSTD_decompress(dest, dest_size, source, source_size);
         Py_END_ALLOW_THREADS
 
         if (ZSTD_isError(cSize)) {
-            PyErr_Format(ZstdError, "Decompression error: %s", ZSTD_getErrorName(cSize));
-            error = 1;
+			const char *errStr = ZSTD_getErrorName(cSize);
+/*			if (strstr(errStr, "buffer is too small") != NULL) {
+				// reroll decompression with bigger buffer
+				error = 2;
+			} else {*/
+            	PyErr_Format(ZstdError, "Decompression error: %s", errStr);
+            	error = 1;
+//			}
         } else if (cSize != dest_size) {
             PyErr_Format(ZstdError, "Decompression error: length mismatch -> decomp %lu != %lu [header]", (uint64_t)cSize, dest_size);
             error = 1;
@@ -172,6 +203,10 @@ static PyObject *py_zstd_uncompress(PyObject* self, PyObject *args)
         result = NULL;
     }
 
+	if (!error) {
+    	Py_SET_SIZE(result, cSize);
+	}
+
     return result;
 }
 
@@ -181,9 +216,9 @@ static PyObject *py_zstd_uncompress(PyObject* self, PyObject *args)
 static PyObject *py_zstd_module_version(PyObject* self, PyObject *args)
 {
 #if PY_MAJOR_VERSION >= 3
-    return PyUnicode_FromFormat("%s", VERSION);
+    return PyUnicode_FromFormat("%s", xstr(VERSION));
 #else
-    return PyString_FromFormat("%s", VERSION);
+    return PyString_FromFormat("%s", xstr(VERSION));
 #endif
 }
 
@@ -216,6 +251,25 @@ static PyObject *py_zstd_library_external(PyObject* self, PyObject *args)
 }
 
 
+/**
+ * Returns ZSTD determined threads count, int
+ */
+static PyObject *py_zstd_threads_count(PyObject* self, PyObject *args)
+{
+    int32_t threads = UTIL_countPhysicalCores();
+
+    return Py_BuildValue("i", threads);
+}
+
+/**
+ * Returns ZSTD determined max threads count, int
+ */
+static PyObject *py_zstd_max_threads_count(PyObject* self, PyObject *args)
+{
+    return Py_BuildValue("i", ZSTDMT_NBWORKERS_MAX);
+}
+
+
 static PyMethodDef ZstdMethods[] = {
     {"ZSTD_compress",  py_zstd_compress_mt, METH_VARARGS, COMPRESS_DOCSTRING},
     {"ZSTD_uncompress",  py_zstd_uncompress, METH_VARARGS, UNCOMPRESS_DOCSTRING},
@@ -227,6 +281,8 @@ static PyMethodDef ZstdMethods[] = {
     {"version",  py_zstd_module_version, METH_NOARGS, VERSION_DOCSTRING},
     {"ZSTD_version",  py_zstd_library_version, METH_NOARGS, ZSTD_VERSION_DOCSTRING},
     {"ZSTD_version_number",  py_zstd_library_version_int, METH_NOARGS, ZSTD_INT_VERSION_DOCSTRING},
+    {"ZSTD_threads_count",  py_zstd_threads_count, METH_NOARGS, ZSTD_THREADS_COUNT_DOCSTRING},
+    {"ZSTD_max_threads_count",  py_zstd_max_threads_count, METH_NOARGS, ZSTD_MAX_THREADS_COUNT_DOCSTRING},
     {"ZSTD_external",  py_zstd_library_external, METH_NOARGS, ZSTD_EXTERNAL_DOCSTRING},
     {NULL, NULL, 0, NULL}
 };
